from time import sleep

from recoverpy.models.search_progress import SearchProgress


def mock_monitor_search_progress(grep_pid: int, progress: SearchProgress) -> None:
    while True:
        progress.progress_percent = 100.0
        sleep(5)



import pytest

from recoverpy import RecoverpyApp


@pytest.mark.asyncio
async def test_not_root(
    mock_not_root, mock_linux, mock_valid_version, mock_dependencies_installed
):
    async with RecoverpyApp().run_test() as p:
        await p.pause()

        assert p.app is not None
        assert p.app.screen.name == "root-error-modal"

        await p.click("#ok-button")
        await p.pause()

        assert p.app.screen.name == "params"


@pytest.mark.asyncio
async def test_invalid_python_version(
    mock_root, mock_linux, mock_invalid_version, mock_dependencies_installed
):
    async with RecoverpyApp().run_test() as p:
        await p.pause()

        assert p.app is not None
        assert p.app.screen.name == "version-error-modal"


@pytest.mark.asyncio
async def test_not_linux(
    mock_root, mock_not_linux, mock_valid_version, mock_dependencies_installed
):
    async with RecoverpyApp().run_test() as p:
        await p.pause()

        assert p.app is not None
        assert p.app.screen.name == "linux-error-modal"

        await p.click("#ok-button")
        await p.pause()

        assert p.app.screen.name == "params", "params"


@pytest.mark.asyncio
async def test_dependencies_not_installed(
    mock_root, mock_linux, mock_valid_version, mock_dependencies_not_installed
):
    async with RecoverpyApp().run_test() as p:
        await p.pause()

        assert p.app is not None
        assert p.app.screen.name == "dependencies-error-modal"

        await p.click("#ok-button")
        await p.pause()

        assert p.app.screen.name == "params"


@pytest.mark.asyncio
async def test_multiple_errors(
    mock_not_root,
    mock_not_linux,
    mock_valid_version,
    mock_dependencies_not_installed,
):
    async with RecoverpyApp().run_test() as p:
        await p.pause()

        assert p.app is not None

        for _ in range(3):
            assert "error" in p.app.screen.name
            await p.click("#ok-button")
            await p.pause()

        assert p.app.screen.name == "params"



from unittest.mock import MagicMock

import pytest
import pytest_asyncio

from recoverpy.lib.search.search_engine import SearchEngine

from .fixtures import (
    mock_dd_output,
    mock_grep_process,
    mock_lsblk_output,
    mock_progress,
)

TEST_BLOCK_SIZE = 4096


@pytest_asyncio.fixture(scope="session", autouse=True)
def system_calls_mock(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.search.search_engine.start_grep_process",
        new=mock_grep_process.mock_start_grep_process,
    )
    session_mocker.patch(
        "recoverpy.lib.search.search_engine.get_dd_output",
        side_effect=mock_dd_output.mock_dd_string_output,
    )
    session_mocker.patch(
        "recoverpy.ui.screens.screen_result.get_dd_output",
        side_effect=mock_dd_output.mock_dd_string_output,
    )
    session_mocker.patch(
        "recoverpy.lib.lsblk._fetch_lsblk_output",
        return_value=mock_lsblk_output.MOCK_LSBLK_OUTPUT,
    )
    session_mocker.patch(
        "recoverpy.models.search_params.get_block_size",
        MagicMock(return_value=TEST_BLOCK_SIZE),
    )
    session_mocker.patch(
        "recoverpy.lib.search.thread_factory.is_dependency_installed",
        MagicMock(return_value=True),
    )
    session_mocker.patch(
        "recoverpy.lib.search.thread_factory.monitor_search_progress",
        new=mock_progress.mock_monitor_search_progress,
    )


@pytest.fixture(scope="session")
def search_engine():
    return SearchEngine(partition="/dev/sda1", searched_string="Lorem ipsum")


@pytest.fixture(scope="function")
def mock_root(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._is_user_root",
        MagicMock(return_value=True),
    )


@pytest.fixture(scope="function")
def mock_not_root(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._is_user_root",
        MagicMock(return_value=False),
    )


@pytest.fixture(scope="function")
def mock_valid_version(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._is_version_supported",
        MagicMock(return_value=True),
    )


@pytest.fixture(scope="function")
def mock_invalid_version(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._is_version_supported",
        MagicMock(return_value=False),
    )


@pytest.fixture(scope="function")
def mock_linux(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._is_linux",
        MagicMock(return_value=True),
    )


@pytest.fixture(scope="function")
def mock_not_linux(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._is_linux",
        MagicMock(return_value=False),
    )


@pytest.fixture(scope="function")
def mock_dependencies_installed(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._are_system_dependencies_installed",
        MagicMock(return_value=True),
    )


@pytest.fixture(scope="function")
def mock_dependencies_not_installed(session_mocker):
    session_mocker.patch(
        "recoverpy.lib.env_check._are_system_dependencies_installed",
        MagicMock(return_value=False),
    )


def pytest_runtest_makereport(item, call):
    if "incremental" in item.keywords and call.excinfo is not None:
        parent = item.parent
        parent._previousfailed = item


def pytest_runtest_setup(item):
    previousfailed = getattr(item.parent, "_previousfailed", None)
    if previousfailed is not None:
        pytest.skip("previous test failed (%s)" % previousfailed.name)



from pathlib import Path
from unittest.mock import MagicMock

import pytest

from recoverpy import RecoverpyApp
from tests.conftest import TEST_BLOCK_SIZE
from tests.fixtures.mock_grep_process import GREP_RESULT_COUNT
from tests.fixtures.mock_lsblk_output import (
    UNFILTERED_PARTITION_COUNT,
    VISIBLE_PARTITION_COUNT,
)
from tests.integration.helper import (
    TEST_FULL_PARTITION,
    TEST_PARTITION,
    add_expected_save_result,
    assert_current_result_is_selected_for_save,
    assert_with_timeout,
    get_block_content_text,
    get_expected_block_content_text,
    get_expected_save_result,
)


@pytest.mark.asyncio
async def test_full_workflow(session_mocker, tmp_path: Path):
    # Apply patches
    session_mocker.patch(
        "recoverpy.lib.env_check._is_user_root",
        MagicMock(return_value=True),
    )
    session_mocker.patch(
        "recoverpy.lib.env_check._are_system_dependencies_installed",
        MagicMock(return_value=True),
    )
    session_mocker.patch(
        "recoverpy.lib.env_check._is_linux",
        MagicMock(return_value=True),
    )

    # Launch the application
    async with RecoverpyApp().run_test() as pilot:
        # Test initialization of the app
        assert pilot.app is not None
        for screen in pilot.app.screens:
            assert pilot.app.is_screen_installed(pilot.app.screens[screen])
        assert pilot.app.screen.name == "params"

        await pilot.pause()
        assert pilot.app.screen._partition_list is not None
        assert (
            len(pilot.app.screen._partition_list.list_items) == VISIBLE_PARTITION_COUNT
        )
        assert pilot.app.screen._start_search_button.disabled is True

        # Test partition list default state
        filter_checkbox = pilot.app.screen.query("Checkbox").only_one()
        assert filter_checkbox is not None
        assert filter_checkbox.value is True

        items = list(pilot.app.screen.query("ListItem").results())
        assert len(items) == VISIBLE_PARTITION_COUNT

        # Test partition list unfiltered
        filter_checkbox.toggle()
        await pilot.pause()

        assert filter_checkbox.value is False
        await assert_with_timeout(
            lambda: len(list(pilot.app.screen.query("ListItem").results()))
            == UNFILTERED_PARTITION_COUNT,
            UNFILTERED_PARTITION_COUNT,
            len(list(pilot.app.screen.query("ListItem").results())),
        )

        # Test partition list filtered
        filter_checkbox.toggle()
        await pilot.pause()

        assert filter_checkbox.value is True
        await assert_with_timeout(
            lambda: len(list(pilot.app.screen.query("ListItem").results()))
            == VISIBLE_PARTITION_COUNT,
            VISIBLE_PARTITION_COUNT,
            len(list(pilot.app.screen.query("ListItem").results())),
        )

        # Test input search parameters
        await pilot.click("#search-input")
        await pilot.pause()

        for c in "TEST":
            await pilot.press(c)

        assert pilot.app.screen._search_input.value == "TEST"

        await pilot.click(f"#{TEST_PARTITION}")
        await pilot.pause()

        assert pilot.app.screen._partition_list.highlighted_child is not None
        assert pilot.app.screen._partition_list.highlighted_child.id == TEST_PARTITION
        assert pilot.app.screen._start_search_button.disabled is False

        # Test start search
        await pilot.click("#start-search-button")
        await pilot.pause()

        assert pilot.app.screen.name == "search"
        assert hasattr(pilot.app.screen, "search_engine")
        assert pilot.app.screen.search_engine.search_params.search_string == "TEST"
        assert (
            pilot.app.screen.search_engine.search_params.partition
            == TEST_FULL_PARTITION
        )

        # Test search results
        await assert_with_timeout(
            lambda: int(pilot.app.screen.search_engine.search_progress.progress_percent)
            == 100,
            100.0,
            pilot.app.screen.search_engine.search_progress.progress_percent,
        )
        await assert_with_timeout(
            lambda: pilot.app.screen.search_engine.search_progress.result_count
            == GREP_RESULT_COUNT,
            GREP_RESULT_COUNT,
            pilot.app.screen.search_engine.search_progress.result_count,
        )
        await assert_with_timeout(
            lambda: len(pilot.app.screen._grep_result_list.grep_results)
            == GREP_RESULT_COUNT,
            GREP_RESULT_COUNT,
            len(pilot.app.screen._grep_result_list.grep_results),
        )
        await assert_with_timeout(
            lambda: len(list(pilot.app.screen.query("ListItem").results()))
            == GREP_RESULT_COUNT,
            GREP_RESULT_COUNT,
            len(list(pilot.app.screen.query("ListItem").results())),
        )

        # Test select search results
        list_items = list(pilot.app.screen.query("ListItem").results())

        for item in list_items:
            await pilot.click(f"#{item.id}")
            await pilot.pause()

            assert item.highlighted is True
            assert pilot.app.screen._get_selected_grep_result().list_item == item

        # Test open result
        select_item = pilot.app.screen._grep_result_list.highlighted_child
        assert select_item is not None
        select_grep_result = pilot.app.screen._get_selected_grep_result()
        assert select_grep_result.list_item == select_item

        await pilot.click("#open-button")
        await pilot.pause()

        assert pilot.app.screen.name == "result"

        assert pilot.app.screen._partition == TEST_FULL_PARTITION
        assert pilot.app.screen._block_size == TEST_BLOCK_SIZE
        assert pilot.app.screen._inode == select_grep_result.inode

        # Test result content
        await pilot.pause()

        assert (
            str(pilot.app.screen._inode)
            in pilot.app.screen._inode_label.renderable
        )
        assert pilot.app.screen._block_count_label.renderable.startswith("0 ")
        assert get_expected_block_content_text(
            pilot.app.screen._inode
        ) == get_block_content_text(pilot.app.screen._block_content)

        # Test select first result
        await pilot.click("#add-block-button")
        await pilot.pause()

        assert pilot.app.screen._block_count_label.renderable.startswith("1 ")
        assert pilot.app.screen._save_button.disabled is False

        assert len(pilot.app.screen._saver._results) == 1
        await assert_current_result_is_selected_for_save(pilot)

        await pilot.click("#add-block-button")
        await pilot.pause()

        assert len(pilot.app.screen._saver._results) == 1
        add_expected_save_result(pilot)

        # Test select next result
        current_inode = pilot.app.screen._inode
        await pilot.click("#next-button")
        await pilot.pause()

        assert pilot.app.screen._inode != current_inode

        assert (
            str(pilot.app.screen._inode)
            in pilot.app.screen._inode_label.renderable
        )

        assert get_expected_block_content_text(
            pilot.app.screen._inode
        ) == get_block_content_text(pilot.app.screen._block_content)

        await pilot.click("#add-block-button")
        await pilot.pause()

        assert pilot.app.screen._block_count_label.renderable.startswith("2 ")

        assert len(pilot.app.screen._saver._results) == 2
        await assert_current_result_is_selected_for_save(pilot)

        await pilot.click("#add-block-button")
        await pilot.pause()

        assert len(pilot.app.screen._saver._results) == 2
        add_expected_save_result(pilot)

        # Test select previous result
        previous_button = pilot.app.screen.query("#previous-button").only_one()
        current_inode = pilot.app.screen._inode
        await pilot.click("#previous-button")
        await pilot.pause()

        assert pilot.app.screen._inode != current_inode

        current_inode = pilot.app.screen._inode

        await assert_with_timeout(
            lambda: not previous_button.has_class("-active"),
            False,
            not previous_button.has_class("-active"),
        )

        await pilot.click("#previous-button")
        await pilot.pause()

        assert pilot.app.screen._inode != current_inode

        assert (
            str(pilot.app.screen._inode)
            in pilot.app.screen._inode_label.renderable
        )
        assert get_expected_block_content_text(
            pilot.app.screen._inode
        ) == get_block_content_text(pilot.app.screen._block_content)

        await pilot.click("#add-block-button")
        await pilot.pause()

        assert pilot.app.screen._block_count_label.renderable.startswith("3 ")

        assert len(pilot.app.screen._saver._results) == 3
        await assert_current_result_is_selected_for_save(pilot)

        await pilot.click("#add-block-button")
        await pilot.pause()

        assert len(pilot.app.screen._saver._results) == 3
        add_expected_save_result(pilot)

        # Test start save process
        await pilot.click("#save-button")
        await pilot.pause()

        assert pilot.app.screen.name == "save"

        # Test edit save path
        await pilot.click("#edit-save-path-button")
        await pilot.pause()

        assert pilot.app.screen.name == "path_edit"

        tmp_tree_node = None
        for n in pilot.app.screen._directory_tree._tree_nodes.values():
            if "tmp" in n._label:
                tmp_tree_node = n
                break

        assert tmp_tree_node is not None
        pilot.app.screen._directory_tree.selected_dir = tmp_path

        await pilot.click("#confirm-button")
        await pilot.pause()

        assert pilot.app.screen.name == "save"
        assert pilot.app.screen._saver.save_path == tmp_path

        # Test save results
        await pilot.click("#save-button")
        await pilot.pause()

        assert pilot.app.screen.name == "save-modal"

        await pilot.click("#ok-button")
        await pilot.pause()

        assert pilot.app.screen.name == "result"

        # Check saved result
        dir_files = list(tmp_path.iterdir())
        assert len(dir_files) == 1
        assert dir_files[0].name.startswith("recoverpy-save-")

        with open(dir_files[0], "r") as f:
            result = f.read()
            assert result == get_expected_save_result()



import asyncio
import inspect

from textual.pilot import Pilot
from textual.widgets import RichLog

from tests.conftest import TEST_BLOCK_SIZE
from tests.fixtures.mock_dd_output import mock_dd_string_output

TEST_PARTITION = "sdb1"
TEST_FULL_PARTITION = "/dev/sdb1"
EXPECTED_SAVE_RESULT = {}


def get_block_content_text(block_content: RichLog):
    return (
        "".join([line.text for line in block_content.lines])
        .replace(" ", "")
        .replace("\n", "")
    )


def get_expected_block_content_text(inode: int):
    return (
        mock_dd_string_output(TEST_FULL_PARTITION, TEST_BLOCK_SIZE, inode)
        .decode()
        .replace(" ", "")
        .replace("\n", "")
    )


async def assert_with_timeout(check_func, expected, actual, timeout=10.0, interval=0.1):
    end_time = asyncio.get_event_loop().time() + timeout
    while True:
        if check_func():
            return
        if asyncio.get_event_loop().time() > end_time:
            func_src = inspect.getsource(check_func).strip()
            assert False, (
                f"Timeout reached before condition in `{func_src}` became true. Expected: {expected}, Actual: {actual}"
            )
        await asyncio.sleep(interval)


async def assert_current_result_is_selected_for_save(p: Pilot):
    await p.pause()

    assert p.app.screen._inode in p.app.screen._saver._results

    assert p.app.screen._saver._results[p.app.screen._inode].replace(" ", "").replace(
        "\n", ""
    ) == get_expected_block_content_text(p.app.screen._inode)


def add_expected_save_result(p: Pilot):
    global EXPECTED_SAVE_RESULT
    EXPECTED_SAVE_RESULT[p.app.screen._inode] = p.app.screen._raw_block_content


def get_expected_save_result():
    global EXPECTED_SAVE_RESULT
    return "\n".join(
        [EXPECTED_SAVE_RESULT[num] for num in sorted(EXPECTED_SAVE_RESULT.keys())]
    )



from os import chdir

from recoverpy.lib.saver import Saver


def test_add(tmp_path):
    saver = Saver()

    saver.add_result(1, "test1")
    saver.add_result(2, "test2")
    saver.add_result(0, "test0")

    assert saver.get_blocks_to_save_count() == 3


def test_reset(tmp_path):
    saver = Saver()

    saver.add_result(1, "test1")
    saver.add_result(2, "test2")
    saver.add_result(0, "test0")
    saver.reset_results()

    assert saver.get_blocks_to_save_count() == 0


def test_update_save_path(tmp_path):
    saver = Saver()

    saver.set_save_path(str(tmp_path))
    assert saver.save_path == tmp_path


def test_get_selected_blocks_count(tmp_path):
    saver = Saver()

    saver.add_result(1, "test1")
    saver.add_result(2, "test2")
    saver.add_result(0, "test0")

    assert saver.get_blocks_to_save_count() == 3


def test_save(tmp_path):
    saver = Saver()

    saver.set_save_path(str(tmp_path))
    saver.add_result(1, "test1")
    saver.add_result(2, "test2")
    saver.add_result(0, "test0")
    saver.save_results()

    assert saver.last_saved_file in list(tmp_path.iterdir())
    with open(saver.last_saved_file, "r") as f:
        assert f.read().splitlines() == ["test0", "test1", "test2"]


def test_save_no_path(tmp_path):
    chdir(tmp_path)
    saver = Saver()

    saver.add_result(1, "test1")
    saver.save_results()

    assert saver.last_saved_file in list(tmp_path.iterdir())



"""A Textual ListView Widget for displaying partitions."""

from typing import Dict, Optional

from textual import work
from textual.events import Mount
from textual.widgets import Label, ListItem, ListView

from recoverpy.lib.lsblk import get_partitions
from recoverpy.log.logger import log
from recoverpy.models.partition import Partition


def _get_label(partition: Partition) -> Label:
    label_content = f"{partition.name} | {partition.fs_type}"
    if partition.is_mounted:
        label_content += f" | Mounted on: {partition.mount_point}"
    return Label(label_content)


def _get_partition_id(partition: Partition) -> str:
    return "".join(filter(str.isalnum, partition.name))


class PartitionList(ListView):
    def __init__(self, *children, **kwargs) -> None:  # type: ignore
        super().__init__(id="partition-list", *children, **kwargs)
        self.list_items: Dict[Optional[str], Partition] = {}

    def _on_mount(self, _: Mount) -> None:
        self.set_partitions()
        return super()._on_mount(_)

    @work(exclusive=True)
    async def set_partitions(self, filtered: bool = True) -> None:
        await self.clear()
        self.list_items.clear()

        for partition in get_partitions(filtered):
            log.debug(f"partition_list - Appending partition {partition.name}")
            list_item = self._create_list_item(partition)
            self.list_items[list_item.id] = partition
            await self.append(list_item)

    def _create_list_item(self, partition: Partition) -> ListItem:
        list_item = ListItem(_get_label(partition), id=_get_partition_id(partition))
        if partition.is_mounted:
            list_item.add_class("mounted")
        return list_item



import pytest

from tests.fixtures.mock_grep_process import GREP_RESULT_COUNT
from tests.integration.helper import assert_with_timeout


def test_search_params(search_engine):
    assert search_engine.search_params.partition == "/dev/sda1"
    assert search_engine.search_params.searched_lines == ["Lorem ipsum"]


def test_search_progress_before_search(search_engine):
    assert search_engine.search_progress.result_count == 0
    assert int(search_engine.search_progress.progress_percent) == 0


@pytest.mark.asyncio
async def test_start_search(search_engine):
    await search_engine.start_search()

    assert search_engine._grep_process.returncode is None


@pytest.mark.asyncio
async def test_search_progress_after_search(search_engine):
    await assert_with_timeout(
        lambda: search_engine.search_progress.result_count == GREP_RESULT_COUNT,
        GREP_RESULT_COUNT,
        search_engine.search_progress.result_count,
    )
    assert int(search_engine.search_progress.progress_percent) == 100


def test_list_items_queue_size(search_engine):
    assert search_engine.formatted_results_queue.qsize() == GREP_RESULT_COUNT


def test_list_items_queue_content(search_engine):
    expected_list_items = [
        [1, "Lorem ipsum dolor sit amet, test consectetur adipiscing elit, sed do "],
        [2, "Lorem ipsum dolor sit amet, test consectetur adipiscing elit, sed da "],
        [3, "Lorem ipsum dolor sit amet, test consectetur adipiscing elit, sed du"],
    ]

    results = []
    while not search_engine.formatted_results_queue.empty():
        results.append(search_engine.formatted_results_queue.get_nowait())

    for i, result in enumerate(results):
        assert expected_list_items[i][0] == result.inode
        assert expected_list_items[i][1] == result.line



from subprocess import PIPE, Popen

GREP_RESULT_COUNT = 3

_MOCK_GREP_OUTPUT = """4096: Lorem ipsum dolor sit amet, test consectetur adipiscing elit, sed do
8192: Lorem ipsum dolor sit amet, test consectetur adipiscing elit, sed da
12288: Lorem ipsum dolor sit amet, test consectetur adipiscing elit, sed du"""


def mock_start_grep_process(*args, **kwargs) -> Popen:
    return Popen(["printf", _MOCK_GREP_OUTPUT], stdin=None, stdout=PIPE, stderr=None)



from recoverpy.lib.lsblk import _IGNORED_PARTITION_TYPES, get_partitions
from tests.fixtures.mock_lsblk_output import (
    UNFILTERED_PARTITION_COUNT,
    VISIBLE_PARTITION_COUNT,
)


def test_get_partitions_filtered():
    partitions = get_partitions(True)

    assert len(partitions) == VISIBLE_PARTITION_COUNT
    for ignored_type in _IGNORED_PARTITION_TYPES:
        assert all(ignored_type not in partition.name for partition in partitions)


def test_get_partitions_unfiltered():
    partitions = get_partitions(False)

    assert len(partitions) == UNFILTERED_PARTITION_COUNT



"""A Textual ListView widget consuming an asyncio Queue"""

from __future__ import annotations

from asyncio import Lock, Queue, sleep
from typing import List, cast

from textual.widgets import Label, ListView

from recoverpy.log.logger import log
from recoverpy.models.grep_result import GrepResult


class GrepResultList(ListView):
    list_items_background_color = {0: "red", 1: "green"}

    def __init__(self, *children, **kwargs) -> None:  # type: ignore
        super().__init__(*children, **kwargs)
        self.results: List[str] = []
        self.lock = Lock()
        self.grep_results: List[GrepResult] = []
        self.id = "grep-result-list"

    async def start_consumer(self, queue: Queue[GrepResult]) -> None:
        log.debug("grep_result_list - Starting consumer")
        while True:
            if not self._should_add_more():
                await sleep(0.1)
                continue

            log.debug("grep_result_list - Adding more results")
            grep_result = await queue.get()
            await self._append(grep_result)

    def get_index(self) -> int:
        return self.index or 0

    async def _append(self, grep_result: GrepResult) -> None:
        log.debug(f"grep_result_list - Appending inode {grep_result.inode}")
        grep_result.create_list_item()

        if not grep_result.list_item:
            log.error(
                f"grep_result_list - Grep result {grep_result.inode} has no list item"
            )
            return

        async with self.lock:
            self.grep_results.append(grep_result)
            new_index = len(self.grep_results) - 1
            self._resize_item(new_index)
            await super().append(grep_result.list_item)

    def _get_list_index_to_show(self) -> int:
        return int(self.size.height) + int(self.scroll_y) + 10

    def _should_add_more(self) -> bool:
        return len(self.children) < self._get_list_index_to_show()

    def on_resize(self) -> None:
        for index in range(len(self.children)):
            self._resize_item(index)

    def _resize_item(self, index: int) -> None:
        max_item_width = self.size.width - self.size.width // 20
        grep_result_line = self.grep_results[index].line
        cast(Label, self.grep_results[index].label).update(
            grep_result_line[:max_item_width]
        )



MOCK_LSBLK_OUTPUT = """loop1 loop squashfs /snap/core18/2284
loop0 loop squashfs /snap/chromium/2238
sda disk
sda1 part ext4 /media/disk1
sdb disk
sdb1 part ntfs /media/disk2
mmcblk0 disk
mmcblk0p1 part vfat
nvme0n1 disk
nvme0n1p1 part ext4 /
system-root lvm btrfs /test
vdb disk LVM2_member
vda2 part LVM2_member"""

VISIBLE_PARTITION_COUNT = 7
UNFILTERED_PARTITION_COUNT = 9
